import {App as BaseApp, AppOperations, BaseAppState, Endpoint, Response} from '../common';
import {DataSource} from './data-source';

interface BackendParams<App extends BaseApp> {
	app: App;
	dataSource?: DataSource;
}

export interface ImplementationContext {
	endpoint: Endpoint;
	params: Record<string, unknown>;
	query?: URLSearchParams;
	dataSource?: DataSource;
}

export type ImplementationFunction = (params: ImplementationContext) => Promise<Response | void>;

export interface Backend<App extends BaseApp = BaseApp> {
	app: App;
	dataSource?: DataSource;
	implementations: Map<string, ImplementationFunction>;
	implementOperation<Operation extends AppOperations<App>>(
		operation: Operation, implementation: ImplementationFunction): this;
}

class BackendInstance<App extends BaseApp> implements Backend<App> {
	readonly app: App;
	readonly dataSource?: DataSource;
	readonly implementations: Map<string, ImplementationFunction>;

	constructor(params: BackendParams<App>) {
		this.app = params.app;
		this.dataSource = params.dataSource;
		this.implementations = new Map<string, ImplementationFunction>();
	}

	implementOperation<Operation extends AppOperations<App>>(
		operation: Operation,
		implementation: ImplementationFunction
	) {
		if (!this.implementations.has(operation)) {
			this.implementations.set(operation, implementation);
		}
		return this;
	}
}

export const backend = <App extends BaseApp<AppName, State>, AppName extends string, State extends BaseAppState = BaseAppState>(params: BackendParams<App>): Backend<App> => {
	return new BackendInstance(params);
};

export class PlainResponse {
	constructor() {
	}
}
