import {QueryAndGrouping, validation as v} from '../common';

export interface EmplaceDetails {
	isCreated: boolean;
}

export type DataSourceQuery = QueryAndGrouping;

export interface DataSource<
	ItemData extends object = object,
	ID extends unknown = unknown,
	Query extends DataSourceQuery = DataSourceQuery,
	Emplace extends EmplaceDetails = EmplaceDetails,
	DeleteResult = unknown,
> {
	initialize(): Promise<unknown>;
	getTotalCount?(query?: Query): Promise<number>;
	getMultiple(query?: Query): Promise<ItemData[]>;
	getById(id: ID): Promise<ItemData | null>;
	getSingle?(query?: Query): Promise<ItemData | null>;
	create(data: ItemData): Promise<ItemData>;
	delete(id: ID): Promise<DeleteResult>;
	emplace(id: ID, data: ItemData): Promise<[ItemData, Emplace]>;
	patch(id: ID, data: Partial<ItemData>): Promise<ItemData | null>;
	newId(): Promise<ID>;
}

export interface ResourceIdConfig<IdSchema extends v.BaseSchema> {
	generationStrategy: GenerationStrategy;
	serialize: (id: unknown) => string;
	deserialize: (id: string) => v.Output<IdSchema>;
	schema: IdSchema;
}

export interface GenerationStrategy {
	(dataSource: DataSource, ...args: unknown[]): Promise<string | number | unknown>;
}
