export const AVAILABLE_METHODS = [
	'HEAD',
	'GET',
	'POST',
	'PUT',
	'PATCH',
	'DELETE',
	'OPTIONS'
] as const;

export const AVAILABLE_EXTENSION_METHODS = [
	'QUERY'
] as const;

export type Method = typeof AVAILABLE_METHODS[number];

export type MethodWithExtensions = Method | typeof AVAILABLE_EXTENSION_METHODS[number];

export interface BaseOperationParams<
	Name extends string = string,
	Method extends MethodWithExtensions = MethodWithExtensions,
> {
	name: Name;
	method?: Method;
}

export interface Operation<Params extends BaseOperationParams = BaseOperationParams> {
	name: Params['name'];
	method: Params['method'];
	searchParams?: URLSearchParams;
	search: (...args: ConstructorParameters<typeof URLSearchParams>) => Operation<Params>;
	setBody: (b: unknown) => Operation<Params>;
	body?: unknown;
}

class OperationInstance<Params extends BaseOperationParams = BaseOperationParams> implements Operation<Params> {
	readonly name: Params['name'];
	readonly method: Params['method'];
	theSearchParams?: URLSearchParams;
	// todo add type safety, depend on method when allowing to have body
	theBody?: unknown;

	constructor(params: Params) {
		this.name = params.name;
		this.method = params.method ?? 'GET';
	}

	search(...args: ConstructorParameters<typeof URLSearchParams>): Operation<Params> {
		this.theSearchParams = new URLSearchParams(...args);
		return this;
	}

	get searchParams() {
		return Object.freeze(this.theSearchParams);
	}

	get body() {
		return Object.freeze(this.theBody);
	}

	setBody(b: unknown): Operation<Params> {
		switch (this.method) {
			case 'PATCH':
			case 'PUT':
			case 'POST':
			case 'QUERY':
				this.theBody = b;
				break;
			default:
				break;
		}

		return this;
	}
}

export const operation = <Params extends BaseOperationParams = BaseOperationParams>(
	params: Params
): Operation<Params> => {
	return new OperationInstance(params);
};
