import {MediaType} from '../media-type';

const OPERATORS = [
	'=',
	'!=',
	'>=',
	'<=',
	'>',
	'<',
	'LIKE',
	'ILIKE',
	'REGEXP',
] as const;

type QueryOperator = typeof OPERATORS[number];

type QueryExpressionValue = string | number | boolean;

interface QueryOperatorExpression {
	lhs: string;
	operator: QueryOperator;
	rhs: QueryExpressionValue | RegExp;
}

interface QueryFunctionExpression {
	name: string;
	args: QueryExpressionValue[];
}

export type QueryAnyExpression = QueryOperatorExpression | QueryFunctionExpression;

export interface QueryOrGrouping {
	type: 'or';
	expressions: QueryAnyExpression[];
}

export interface QueryAndGrouping {
	type: 'and';
	expressions: QueryOrGrouping[];
}

export type Query = QueryAndGrouping;

export interface QueryMediaType<
	Name extends string = string,
	SerializeOptions extends {} = {},
	DeserializeOptions extends {} = {}
> extends MediaType<
	Name,
	QueryAndGrouping,
	SerializeOptions,
	DeserializeOptions
> {}
