import {Backend, DataSource} from '../backend';
import {Operation} from './operation';
import {App} from './app';
import {Endpoint} from './endpoint';

export interface RecipeState<AppName extends string = string, A extends App<AppName> = App<AppName>> {
	app: A;
	backend?: Backend<A>;
	dataSource?: DataSource;
	operations?: Record<string, Operation>;
	endpoints?: Record<string, Endpoint>;
}

export type Recipe<SA extends string = string, A extends App<SA> = App<SA>, B extends A = A> = (a: RecipeState<SA, A>) => RecipeState<SA, B>;

export const composeRecipes = (recipes: Recipe[]): Recipe => (params) => (
	recipes.reduce(
		(rr, r) => r(rr),
		params
	)
);
