// https://github.com/prettymuchbryce/http-status-codes

export const HTTP_STATUS_CONTINUE = 100 as const;
export const HTTP_STATUS_SWITCHING_PROTOCOLS = 101 as const;
export const HTTP_STATUS_PROCESSING = 102 as const;
export const HTTP_STATUS_EARLY_HINTS = 103 as const;

export const IN_TRANSIT_STATUS_CODES = [
	HTTP_STATUS_CONTINUE,
	HTTP_STATUS_SWITCHING_PROTOCOLS,
	HTTP_STATUS_PROCESSING,
	HTTP_STATUS_EARLY_HINTS,
] as const;

export type InTransitStatusCode = typeof IN_TRANSIT_STATUS_CODES[number];

export const isInTransitStatusCode = (statusCode: StatusCode): statusCode is InTransitStatusCode => (
	IN_TRANSIT_STATUS_CODES.includes(statusCode as InTransitStatusCode)
);

export const HTTP_STATUS_OK = 200 as const;
export const HTTP_STATUS_CREATED = 201 as const;
export const HTTP_STATUS_ACCEPTED = 202 as const;
export const HTTP_STATUS_NON_AUTHORITATIVE_INFORMATION = 203 as const;
export const HTTP_STATUS_NO_CONTENT = 204 as const;
export const HTTP_STATUS_RESET_CONTENT = 205 as const;
export const HTTP_STATUS_PARTIAL_CONTENT = 206 as const;
export const HTTP_STATUS_MULTI_STATUS = 207 as const;
export const HTTP_STATUS_ALREADY_REPORTED = 208 as const;
export const HTTP_STATUS_IM_USED = 226 as const;

export const RESOLVED_STATUS_CODES = [
	HTTP_STATUS_OK,
	HTTP_STATUS_CREATED,
	HTTP_STATUS_ACCEPTED,
	HTTP_STATUS_NON_AUTHORITATIVE_INFORMATION,
	HTTP_STATUS_NO_CONTENT,
	HTTP_STATUS_RESET_CONTENT,
	HTTP_STATUS_PARTIAL_CONTENT,
	HTTP_STATUS_MULTI_STATUS,
	HTTP_STATUS_ALREADY_REPORTED,
	HTTP_STATUS_IM_USED,
] as const;

export type ResolvedStatusCode = typeof RESOLVED_STATUS_CODES[number];

export const isResolvedStatusCode = (statusCode: StatusCode): statusCode is ResolvedStatusCode => (
	RESOLVED_STATUS_CODES.includes(statusCode as ResolvedStatusCode)
);

export const HTTP_STATUS_MULTIPLE_CHOICES = 300 as const;
export const HTTP_STATUS_MOVED_PERMANENTLY = 301 as const;
export const HTTP_STATUS_FOUND = 302 as const;
export const HTTP_STATUS_SEE_OTHER = 303 as const;
export const HTTP_STATUS_NOT_MODIFIED = 304 as const;
export const HTTP_STATUS_USE_PROXY = 305 as const;
export const HTTP_STATUS_TEMPORARY_REDIRECT = 307 as const;
export const HTTP_STATUS_PERMANENT_REDIRECT = 308 as const;

export const REDIRECT_STATUS_CODES = [
	HTTP_STATUS_MULTIPLE_CHOICES,
	HTTP_STATUS_MOVED_PERMANENTLY,
	HTTP_STATUS_FOUND,
	HTTP_STATUS_SEE_OTHER,
	HTTP_STATUS_NOT_MODIFIED,
	HTTP_STATUS_USE_PROXY,
	HTTP_STATUS_TEMPORARY_REDIRECT,
	HTTP_STATUS_PERMANENT_REDIRECT,
] as const;

export type RedirectStatusCode = typeof REDIRECT_STATUS_CODES[number];

export const isRedirectStatusCode = (statusCode: StatusCode): statusCode is RedirectStatusCode => (
	REDIRECT_STATUS_CODES.includes(statusCode as RedirectStatusCode)
);

export const SUCCESS_STATUS_CODES = [
	...RESOLVED_STATUS_CODES,
	...REDIRECT_STATUS_CODES,
];

export type SuccessStatusCode = typeof SUCCESS_STATUS_CODES[number];

export const isSuccessStatusCode = (statusCode: StatusCode): statusCode is SuccessStatusCode => (
	SUCCESS_STATUS_CODES.includes(statusCode as SuccessStatusCode)
);

export const HTTP_STATUS_BAD_REQUEST = 400 as const;
export const HTTP_STATUS_UNAUTHORIZED = 401 as const;
export const HTTP_STATUS_PAYMENT_REQUIRED = 402 as const;
export const HTTP_STATUS_FORBIDDEN = 403 as const;
export const HTTP_STATUS_NOT_FOUND = 404 as const;
export const HTTP_STATUS_METHOD_NOT_ALLOWED = 405 as const;
export const HTTP_STATUS_NOT_ACCEPTABLE = 406 as const;
export const HTTP_STATUS_PROXY_AUTHENTICATION_REQUIRED = 407 as const;
export const HTTP_STATUS_REQUEST_TIMEOUT = 408 as const;
export const HTTP_STATUS_CONFLICT = 409 as const;
export const HTTP_STATUS_GONE = 410 as const;
export const HTTP_STATUS_LENGTH_REQUIRED = 411 as const;
export const HTTP_STATUS_PRECONDITION_FAILED = 412 as const;
export const HTTP_STATUS_PAYLOAD_TOO_LARGE = 413 as const;
export const HTTP_STATUS_URI_TOO_LONG = 414 as const;
export const HTTP_STATUS_UNSUPPORTED_MEDIA_TYPE = 415 as const;
export const HTTP_STATUS_RANGE_NOT_SATISFIABLE = 416 as const;
export const HTTP_STATUS_EXPECTATION_FAILED = 417 as const;
export const HTTP_STATUS_TEAPOT = 418 as const;
export const HTTP_STATUS_INSUFFICIENT_SPACE_ON_RESOURCE = 419 as const;
export const HTTP_STATUS_METHOD_FAILURE = 420 as const;
export const HTTP_STATUS_MISDIRECTED_REQUEST = 421 as const;
export const HTTP_STATUS_UNPROCESSABLE_ENTITY = 422 as const;
export const HTTP_STATUS_LOCKED = 423 as const;
export const HTTP_STATUS_FAILED_DEPENDENCY = 424 as const;
export const HTTP_STATUS_TOO_EARLY = 425 as const;
export const HTTP_STATUS_UPGRADE_REQUIRED = 426 as const;
export const HTTP_STATUS_PRECONDITION_REQUIRED = 428 as const;
export const HTTP_STATUS_TOO_MANY_REQUESTS = 429 as const;
export const HTTP_STATUS_REQUEST_HEADER_FIELDS_TOO_LARGE = 431 as const;
export const HTTP_STATUS_UNAVAILABLE_FOR_LEGAL_REASONS = 451 as const;

export const CLIENT_ERROR_STATUS_CODES = [
	HTTP_STATUS_BAD_REQUEST,
	HTTP_STATUS_UNAUTHORIZED,
	HTTP_STATUS_PAYMENT_REQUIRED,
	HTTP_STATUS_FORBIDDEN,
	HTTP_STATUS_NOT_FOUND,
	HTTP_STATUS_METHOD_NOT_ALLOWED,
	HTTP_STATUS_NOT_ACCEPTABLE,
	HTTP_STATUS_PROXY_AUTHENTICATION_REQUIRED,
	HTTP_STATUS_REQUEST_TIMEOUT,
	HTTP_STATUS_CONFLICT,
	HTTP_STATUS_GONE,
	HTTP_STATUS_LENGTH_REQUIRED,
	HTTP_STATUS_PRECONDITION_FAILED,
	HTTP_STATUS_PAYLOAD_TOO_LARGE,
	HTTP_STATUS_URI_TOO_LONG,
	HTTP_STATUS_UNSUPPORTED_MEDIA_TYPE,
	HTTP_STATUS_RANGE_NOT_SATISFIABLE,
	HTTP_STATUS_EXPECTATION_FAILED,
	HTTP_STATUS_TEAPOT,
	HTTP_STATUS_INSUFFICIENT_SPACE_ON_RESOURCE,
	HTTP_STATUS_METHOD_FAILURE,
	HTTP_STATUS_MISDIRECTED_REQUEST,
	HTTP_STATUS_UNPROCESSABLE_ENTITY,
	HTTP_STATUS_LOCKED,
	HTTP_STATUS_FAILED_DEPENDENCY,
	HTTP_STATUS_TOO_EARLY,
	HTTP_STATUS_UPGRADE_REQUIRED,
	HTTP_STATUS_PRECONDITION_REQUIRED,
	HTTP_STATUS_TOO_MANY_REQUESTS,
	HTTP_STATUS_REQUEST_HEADER_FIELDS_TOO_LARGE,
	HTTP_STATUS_UNAVAILABLE_FOR_LEGAL_REASONS,
] as const;

export type ClientErrorStatusCode = typeof CLIENT_ERROR_STATUS_CODES[number];

export const isClientErrorStatusCode = (statusCode: StatusCode): statusCode is ClientErrorStatusCode => (
	CLIENT_ERROR_STATUS_CODES.includes(statusCode as ClientErrorStatusCode)
);

export const HTTP_STATUS_INTERNAL_SERVER_ERROR = 500 as const;
export const HTTP_STATUS_NOT_IMPLEMENTED = 501 as const;
export const HTTP_STATUS_BAD_GATEWAY = 502 as const;
export const HTTP_STATUS_SERVICE_UNAVAILABLE = 503 as const;
export const HTTP_STATUS_GATEWAY_TIMEOUT = 504 as const;
export const HTTP_STATUS_HTTP_VERSION_NOT_SUPPORTED = 505 as const;
export const HTTP_STATUS_VARIANT_ALSO_NEGOTIATES = 506 as const;
export const HTTP_STATUS_INSUFFICIENT_STORAGE = 507 as const;
export const HTTP_STATUS_LOOP_DETECTED = 508 as const;
export const HTTP_STATUS_BANDWIDTH_LIMIT_EXCEEDED = 509 as const;
export const HTTP_STATUS_NOT_EXTENDED = 510 as const;
export const HTTP_STATUS_NETWORK_AUTHENTICATION_REQUIRED = 511 as const;

export const SERVER_ERROR_STATUS_CODES = [
	HTTP_STATUS_INTERNAL_SERVER_ERROR,
	HTTP_STATUS_NOT_IMPLEMENTED,
	HTTP_STATUS_BAD_GATEWAY,
	HTTP_STATUS_SERVICE_UNAVAILABLE,
	HTTP_STATUS_GATEWAY_TIMEOUT,
	HTTP_STATUS_HTTP_VERSION_NOT_SUPPORTED,
	HTTP_STATUS_VARIANT_ALSO_NEGOTIATES,
	HTTP_STATUS_INSUFFICIENT_STORAGE,
	HTTP_STATUS_LOOP_DETECTED,
	HTTP_STATUS_BANDWIDTH_LIMIT_EXCEEDED,
	HTTP_STATUS_NOT_EXTENDED,
	HTTP_STATUS_NETWORK_AUTHENTICATION_REQUIRED,
] as const;

export type ServerErrorStatusCode = typeof SERVER_ERROR_STATUS_CODES[number];

export const isServerErrorStatusCode = (statusCode: StatusCode): statusCode is ServerErrorStatusCode => (
	SERVER_ERROR_STATUS_CODES.includes(statusCode as ServerErrorStatusCode)
);

export const ERROR_STATUS_CODES = [
	...CLIENT_ERROR_STATUS_CODES,
	...SERVER_ERROR_STATUS_CODES,
] as const;

export type ErrorStatusCode = typeof ERROR_STATUS_CODES[number];

export const isErrorStatusCode = (statusCode: StatusCode): statusCode is ErrorStatusCode => (
	ERROR_STATUS_CODES.includes(statusCode as ErrorStatusCode)
);

export type StatusCode = InTransitStatusCode | SuccessStatusCode | ErrorStatusCode;
