import {ImplementationFunction, DataSource} from '@modal-sh/yasumi/backend';
import {operation as defineOperation} from '@modal-sh/yasumi';
import {
	DataSourceNotFoundResponseError,
	ItemNotFoundReponseError,
	ResourceCollectionFetchedResponse,
	ResourceItemFetchedResponse,
} from '../response';

export const name = 'fetch' as const;

export const method = 'GET' as const;

export const operation = defineOperation({
	name,
	method,
});

export const implementation: ImplementationFunction = async (ctx) => {
	const dataSource: DataSource = ctx.endpoint.dataSource ?? ctx.dataSource ?? {} as DataSource;
	// need to genericise the response here so we don't depend on the HTTP responses.
	const { resourceId } = ctx.params;
	const { getById, getMultiple } = dataSource;

	if (typeof resourceId === 'undefined') {
		if (typeof getMultiple === 'undefined') {
			throw new DataSourceNotFoundResponseError();
		}

		// TODO add query here
		const items = await getMultiple();
		return new ResourceCollectionFetchedResponse({
			statusMessage: 'Resource Collection Fetched',
			body: items,
		});
	}

	if (typeof getById === 'undefined') {
		throw new DataSourceNotFoundResponseError();
	}

	const item = await getById(resourceId);
	if (!item) {
		throw new ItemNotFoundReponseError();
	}

	return new ResourceItemFetchedResponse({
		statusMessage: 'Resource Item Fetched',
		body: item,
	});
};
